<?php
namespace Controller;

use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Service\ColorPalette as ColorPaletteService;

class ColorPalette extends Crud {
	protected $dataSourceName = 'color';
    protected $mongoManager;
    protected $colorService;

    public function __construct(Manager $mongoManager, ColorPaletteService $colorService){
        $this->mongoManager = $mongoManager;
        $this->colorService = $colorService;
	}

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $emptyIfNull = Utilities::getValueInFilterDataSourceByName($conditions, 'emptyIfNull', true);
        $filter = new FilterCriteria('color',$conditions);
        $colors = $this->colorService->find($filter, $emptyIfNull);
        $dataSet = new DataSet('findColor', $colors);
        $response->setDataSets(array($dataSet));
    }
    
    public function save(DTO\Request $request, DTO\Response $response){
        $colors = $this->mongoManager->findBy(new FilterCriteria('color'))->getRows();
        $row = $request->getRow();
        $this->colorService->save($row, $colors);
    }

    public function delete(DTO\Request $request, DTO\Response $response){
        $colors = $this->mongoManager->findBy(new FilterCriteria('color'))->getRows();
        $row = $request->getRow();
        $this->colorService->delete($row, $colors);
    }
}
