<?php
namespace Controller;

use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Service\ImportDefault as ImportDefaultService;

class ImportDefault extends Crud {
    protected $mongoManager;
    protected $importService;

    public function __construct(){

        $mongoDbName = InstanceManager::getInstance()->getParameter('mongoDBNamePrefix');
        $host = InstanceManager::getInstance()->getParameter('mongoHost');
        $port = InstanceManager::getInstance()->getParameter('mongoPort');
        $baseConnections = array(array(
            'host' => $host,
            'port' => $port,
            'username' => 'zmartzeedhi',
            'password' => 'zmartzeedhi',
            'dbName' => $mongoDbName . '5',
            'dbLabel' => 'Teknisa'
        ));
        Utilities::getEnvironment()->setBaseConnection($baseConnections);

        $this->mongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
        $this->importService = InstanceManager::getInstance()->getService('\Service\ImportDefault');
	}

    public function findImport(DTO\Request\Filter $request, DTO\Response $response) {
        $importData = $this->importService->findImportData();
        $response->addDataSet(new DataSet("findImport", $importData));
    }

    public function importDefault(DTO\Request $request, DTO\Response $response){
        $rows = $request->getRow();
        $accessProfiles = !empty($rows['accessProfiles']) ? $rows["accessProfiles"] : null;
        $row = $rows["rows"];
        list($importedItems,$errorMessages) = $this->importService->importDefault($row, $accessProfiles);
        $response->addDataSet(new DataSet('importDefault', $importedItems));
        $this->importService->setLogImport($importedItems['dashboard'],$importedItems['indicator'],$importedItems['dataSource']);
        foreach($importedItems as $itemName => $itemCount){
            if(count($itemCount) > 0){
                $response->addNotification(new DTO\Response\Notification(Exception::buildErrorJSON(
                    "PERSISTED_SUCCESSFULLY",
                    array("count" => count($itemCount), "name" => $itemName)
                )));
            }
        }
        foreach($errorMessages as $error){
            $response->addNotification(new DTO\Response\Notification($error,"error"));
        }
    }
    
    public function findImportIndicatorViews(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $indicator  = Utilities::getValueInFilterDataSourceByName($conditions, 'indicator', true);
        $importData = $this->importService->findImportIndicatorViews($indicator);
        $response->addDataSet(new DataSet("findImportIndicatorViews", $importData));
    }
}
