<?php
namespace Service;

use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;

class ColorPalette {

    protected $environment;
    protected $manager;

    const DEFAULT_COLORS = [
        '#7ac95b',
        '#f2ca27',
        '#ff7921',
        '#e74c3c',
        '#1ac3d6',
        '#5eb43c',
        '#f39c12',
        '#d35400',
        '#d62d1a',
        '#3498db',
        '#9b59b6',
        '#ff4c65',
        '#34495e',
        '#afb6bb',
        '#217dbb',
        '#804399',
        '#ff334f',
        '#22303d'
    ];

    public function __construct(Manager $manager) {
        $this->manager = $manager;
    }

    private function rgbToHexColor($color){
        return "#".str_pad(dechex($color['_r']), 2, "0", STR_PAD_LEFT) . str_pad(dechex($color['_g']), 2, "0", STR_PAD_LEFT) . str_pad(dechex($color['_b']), 2, "0", STR_PAD_LEFT);
    }

    public function delete($row, $colors){
        $delete = array();
        foreach($colors as $i => $c){
            if($c['_id'] == $row['_id']){
                $delete = $c;
            }
        }
        $this->manager->delete(new DataSet('color', array($delete)));
    }

    public function save($row, $colors){
        $color = $row['color'];
        $order = $row['order'];
        if(is_array($color) && isset($row['colorHex'])){
            $color = $row['colorHex'];
        }else if(is_array($color) && array_key_exists('_r',$color)){
            $color = $this->rgbToHexColor($color);
        }
        if(isset($row['_id'])){
            foreach($colors as $i => $c){
                if($c['_id'] == $row['_id']){
                    $colors[$i]['color'] = $color;
                    $colors[$i]['order'] = $order;
                }
            }
        }else{
            array_push($colors,array("color" => $color,"order" => $order));
        }
        $dataSet = new DataSet('color',$colors);
        $this->manager->persist($dataSet);
    }

    public function find(FilterCriteria $filter, $emptyIfNull = false, $alwaysBringDefaultColors = false) {
        $filter->addOrderBy('order', FilterCriteria::ORDER_ASC);
        $colors = $this->manager->findBy($filter)->getRows();
        if((empty($colors) && !$emptyIfNull)||(!empty($colors) && $alwaysBringDefaultColors)){
            $default = $this::DEFAULT_COLORS;
            foreach($default as $d){
                array_push($colors,array("color" => $d));
            }  
        }
        return $colors;
    }
        
}