<?php

namespace Service;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class DefaultColumns
{

    protected $dataSourceName = "defaultColumns";

    /** @var Manager*/
    protected $mongoManager;

    public function __construct(Manager $mongoManager)
    {
        $this->mongoManager = $mongoManager;
    }

    public function getDefaultColumns($params)
    {
        $filters = $this->validateFilters($params);
        $filterData = new FilterCriteria($this->dataSourceName, $filters);
        $filterData->setDataSourceName($this->dataSourceName);
        return $this->mongoManager->findBy($filterData)->getRows();
    }

    private function validateFilters($params)
    {
        $filters = array();
        if (isset($params['search'])) {
            $searchIn = $this->concatenateFilterField($params['search_in']);
            array_push($filters, array(
                "columnName" => $searchIn,
                "value" => $params['search'],
                "operator" => FilterCriteria::LIKE_ALL
            ));
        }

        if (isset($params['in'])) {
            if (is_array($params['in'])) {
                foreach ($params['in'] as $key => $value) {
                    array_push($filters, array(
                        "columnName" => $value,
                        "value" => $params[$value],
                        "operator" => is_bool($params[$value]) ? FilterCriteria::EQ : FilterCriteria::LIKE_I
                    ));
                }
            } else {
                array_push($filters, array(
                    "columnName" => $params['in'],
                    "value" => $params[$params['in']],
                    "operator" => is_bool($params[$params['in']]) ? FilterCriteria::EQ : FilterCriteria::LIKE_I
                ));
            }
        }

        return $filters;
    }

    private function concatenateFilterField($filterField)
    {
        $concatenatedString = '';
        foreach ($filterField as $key => $value) {
            if ($key === 0) $concatenatedString = $concatenatedString . $value;
            else $concatenatedString = $concatenatedString . '|' . $value;
        }

        return $concatenatedString;
    }

    public function dataSetSave(DataSet $dataSet)
    {
        $dataSet->setDataSourceName($this->dataSourceName);
        return $this->mongoManager->persist($dataSet);
    }

    public function filterFind(FilterCriteria $filterCriteria)
    {
        $filterCriteria->setDataSourceName($this->dataSourceName);
        return $this->mongoManager->findBy($filterCriteria);
    }

    public function findByName($name)
    {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('name', FilterCriteria::EQ, $name);
        return $this->filterFind($filterCriteria);
    }

    public function findByNames(array $names)
    {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('name', FilterCriteria::IN, $names);
        return $this->filterFind($filterCriteria);
    }

    public function insertNewDefault(array $column)
    {
        $dataSet = $this->findByName($column['name']);
        $oldColumns = $dataSet->getRows();
        if (empty($oldColumns)) {
            return $this->dataSetSave(new DataSet($this->dataSourceName, array($column)));
        }
    }

    public function updateOrInsertDefaultColumn(array $column)
    {
        $columnWithId = $this->setDefaultColumnIdIfExists($column);
        return $this->dataSetSave(new DataSet($this->dataSourceName, array($columnWithId)));
    }

    /**
     * @param $column
     * @return $column with _id if exists a default column with the same name
     */
    protected function setDefaultColumnIdIfExists($column)
    {
        $dataSet = $this->findByName($column['name']);
        $oldColumns = $dataSet->getRows();
        if (!empty($oldColumns)) {
            $oldColumn = $oldColumns[0];
            $column['_id'] = $oldColumn['_id'];
        }
        return $column;
    }
}
