<?php
namespace Service;

use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Service\Widget;
use Service\Manager\Mongo\Types\Type;

class PeriodObservation {

    protected $environment;
    protected $manager;
    protected $widgetService;
    protected $dataSourceName = "periodObservation";

    public function __construct(Manager $manager, Widget $widgetService) {
        $this->manager = $manager;
        $this->widgetService = $widgetService;
    }

    public function delete($row){
        $this->manager->delete(new DataSet($this->dataSourceName, [$row]));
    }

    public function save($id, $name, $period, $indicator, $dataSource){
        $observations = array(
            "name" => $name,
            "indicator" => $indicator,
            "dataSource" => $dataSource,
            "period" => array(
                "start" => $this->convertDate($period["start"]),
                "end" => $this->convertDate($period["end"])
            )
        );
        if(!empty($id)){
            $observations["_id"] = $id;
        }
        $dataSet = new DataSet($this->dataSourceName,[$observations]);
        $this->manager->persist($dataSet);
    }

    public function find($params) {
        //$dynamic = base64_decode($params["dynamic_filter"]);
        //var_dump(json_decode($dynamic, true));
        $filter = new ZmartFilterCriteria($this->dataSourceName, array());
        // Order
        if (array_key_exists("order", $params)) {
            foreach ($params["order"] as $order) {
                $orderSplit = explode(".", $order);
                $filter->addOrderBy($orderSplit[0], $orderSplit[1]==="desc"? FilterCriteria::ORDER_DESC : FilterCriteria::ORDER_ASC);
            }
        }
        // Search ALL
        if (array_key_exists("search_in", $params)) {
            $searchValues = $params["search_in"];
            $filter->addCondition(implode('|',$searchValues), FilterCriteria::LIKE_ALL, "%".$params["search"]."%");
        }
        // Filter by field
        if (array_key_exists("in", $params)) {
            foreach ($params["in"] as $paramFilter){
                if(!array_key_exists($paramFilter, $params))
                    continue;
                $operator = FilterCriteria::EQ;
                $filterValue = $params[$paramFilter];
                if (is_string($filterValue)){
                    $filter->addCondition($paramFilter, FilterCriteria::LIKE_I, $filterValue);
                }
                else if (is_array($filterValue) && count($filterValue) > 0){
                    if ($paramFilter == "period")
                        $filter->addCondition($paramFilter, FilterCriteria::BETWEEN, $filterValue);
                    else
                        foreach ($filterValue as $value)
                            $filter->addCondition($paramFilter, FilterCriteria::IN, $value);
                } else {
                    $filter->addCondition($paramFilter, $filterValue);
                }
            }
        }
        return $this->manager->findBy($filter)->getRows();
    }

    private function resetFilterCriteria($id,$condition){
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition($condition,FilterCriteria::EQ,$id);
        return $filterCriteria;
    }

    private function convertDate($date){
        return Type::getType('date')->convertToDatabaseValue(\DateTime::createFromFormat('d/m/Y',$date)->format("Y/m/d"));
    }

    public function getObservationsByDateAndIndicatorAndDataSource($allPeriods,$indicatorId,$dataSourceId){
        $ids = array($indicatorId,$dataSourceId);
        $condition = array('indicator','dataSource');
        $observations = null;
        foreach($ids as $i => $id){
            if(empty($allPeriods[0])){
                $allPeriods = [$allPeriods];
            }
            foreach($allPeriods as $period){
                $filterCriteria = $this->resetFilterCriteria($id,$condition[$i]);
                $indicatorPeriod = null;
                if(!empty($period) && !empty($period["start"])){
                    $indicatorPeriod = array($this->convertDate($period["start"]),$this->convertDate($period["end"]));
                    $filterCriteria->addCondition("period.start",FilterCriteria::BETWEEN,$indicatorPeriod);
                }
                if(is_array($observations)){
                    Utilities::array_push_array($observations,$this->find($filterCriteria));
                } else {
                    $observations = $this->find($filterCriteria);
                }
                if(!empty($indicatorPeriod)){
                    $filterCriteria = $this->resetFilterCriteria($id,$condition[$i]);
                    $filterCriteria->addCondition("period.end",FilterCriteria::BETWEEN,$indicatorPeriod);
                    Utilities::array_push_array($observations,$this->find($filterCriteria));

                    $filterCriteria = $this->resetFilterCriteria($id,$condition[$i]);
                    $filterCriteria->addCondition("period.start",FilterCriteria::LTE,$indicatorPeriod[0]);
                    $filterCriteria->addCondition("period.end",FilterCriteria::GTE,$indicatorPeriod[1]);
                    Utilities::array_push_array($observations,$this->find($filterCriteria));
                }
            }
        }
        $observations = $this->buildObservations($this->uniqueObservations($observations));
        return $observations;
    }

    private function uniqueObservations($observations){
        $uniqueObservations = [];
        $idList = [];
        foreach($observations as $obs){
            if(!in_array($obs["_id"],$idList)){
                array_push($idList,$obs["_id"]);
                array_push($uniqueObservations,$obs);
            }
        }
        return $uniqueObservations;
    }

    private function buildObservations($observations){
        $builtObservations = array();
        foreach($observations as $obs){
            array_push($builtObservations,$obs["name"]." de ".$obs["period"]["start"]->format('d/m/Y')." até ".$obs["period"]["end"]->format('d/m/Y'));
        }
        return $builtObservations;
    }
}