CREATE OR REPLACE FUNCTION APURA_DATA_COMPETENCIA(
   P_NRSEQUMAPAPU    IN MAPAAPURAC.NRSEQUMAPAPU%TYPE,
   P_CDEMPRESA       IN MAPAAPURAC.CDEMPRESA%TYPE,
   P_CDIMPOSTO       IN MAPAAPURAC.CDIMPOSTO%TYPE,
   P_INSCRICAO       IN MAPAAPURAC.CDINSCESTA%TYPE,
   P_DTFINAL         IN MAPAAPURAC.DTFIMPERIODO%TYPE,
   P_DTINICIAL       IN MAPAAPURAC.DTINIPERIODO%TYPE,
   P_BOIMPOSAPUR     IN BOOLEAN,
   P_IDORGARRECADNFE IN IMPOSTONFE.IDORGARRECADNFE%TYPE,
   P_DTININFFORPER   IN PARAMFISCAL.DTININFFORPER%TYPE,
   P_IDNFFORAPERAPU  IN IMPOSTO.IDNFFORAPERAPU%TYPE) RETURN VARCHAR
AS
   V_IDAPURACAO            VARCHAR2(01);
   V_NFFORAPERIODO         VARCHAR2(01);
   V_IDUTCARGATRIB         VARCHAR2(01);
   V_IDCREDDEBNORM         VARCHAR2(01);
   V_IDOUTCREDDEB          VARCHAR2(01);
   V_VRCARGSAID            NUMBER;
   V_VRCARGENTR            NUMBER;
   V_NRSEQITMAPR           NUMBER;
   V_CDAJUSTEITSEF_DEBI    VARCHAR2(03);
   V_CDAJUSTEITSEF_CRED    VARCHAR2(03);
   V_CDITPROCEDENCIA_DEBI  VARCHAR2(10);
   V_CDITPROCEDENCIA_CRED  VARCHAR2(10);
   V_DSOUTDEBICARGA        VARCHAR2(250);
   V_DSOUTCREDCARGA        VARCHAR2(250);
   V_CDAJSTDEBICARGA       VARCHAR2(10);
   V_CDAJSTCREDCARGA       VARCHAR2(10);
   V_UTCARGATRIB           BOOLEAN;   
BEGIN
   
   V_IDAPURACAO  := NULL;
   V_UTCARGATRIB := FALSE;

   /******** Cd. Ajuste SEF e Item Procedncia do Registro de Outros Crditos - Reapurao de Mapas ********/
   BEGIN
      SELECT MAX(CDAJUSTEITSEF), MAX(CDITPROCEDENCIA)
        INTO V_CDAJUSTEITSEF_CRED, V_CDITPROCEDENCIA_CRED
        FROM ITMAPARECUP
       WHERE NRSEQUMAPAPU      = P_NRSEQUMAPAPU
         AND IDTIPOITMAPR      = 'OC'
         AND IDCARGATRIBUTARIA = 'S';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CDAJUSTEITSEF_CRED   := NULL;
         V_CDITPROCEDENCIA_CRED := NULL;
   END;

   /******** Cd. Ajuste SEF e Item Procedncia do Registro de Outros Dbitos - Reapurao de Mapas ********/
   BEGIN
      SELECT MAX(CDAJUSTEITSEF), MAX(CDITPROCEDENCIA)
        INTO V_CDAJUSTEITSEF_DEBI, V_CDITPROCEDENCIA_DEBI
        FROM ITMAPARECUP
       WHERE NRSEQUMAPAPU      = P_NRSEQUMAPAPU
         AND IDTIPOITMAPR      = 'OD'
         AND IDCARGATRIBUTARIA = 'S';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CDAJUSTEITSEF_DEBI   := NULL;
         V_CDITPROCEDENCIA_DEBI := NULL;
   END;

   /******** Descrio e Cd. Ajuste de Outros Crditos / Dbitos ********/
   BEGIN
      SELECT I.IDUTCARGATRIB,   I.IDCREDDEBNORM,   I.IDOUTCREDDEB, I.DSOUTDEBICARGA,
             I.CDAJSTDEBICARGA, I.CDAJSTCREDCARGA, I.DSOUTCREDCARGA
        INTO V_IDUTCARGATRIB,   V_IDCREDDEBNORM,   V_IDOUTCREDDEB,  V_DSOUTDEBICARGA,
             V_CDAJSTDEBICARGA, V_CDAJSTCREDCARGA, V_DSOUTCREDCARGA
        FROM MAPAAPURAC M, INSCRESTAD I
       WHERE M.NRSEQUMAPAPU = P_NRSEQUMAPAPU
         AND M.CDEMPRESA    = P_CDEMPRESA
         AND M.CDINSCESTA   = P_INSCRICAO
         AND M.CDEMPRESA    = I.CDEMPRESA
         AND M.CDINSCESTA   = I.CDINSCESTA;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDUTCARGATRIB   := 'N';
         V_IDCREDDEBNORM   := 'N';
         V_IDOUTCREDDEB    := 'N';
         V_DSOUTDEBICARGA  := NULL;
         V_CDAJSTDEBICARGA := 0;
         V_CDAJSTCREDCARGA := 0;
         V_DSOUTCREDCARGA  := 0;
   END;

   /******** Deleta os registros de Carga Tributria de Outros Crditos/Dbitos - Reapurao de Mapas ********/
   DELETE FROM ITMAPARECUP
    WHERE NRSEQUMAPAPU      = P_NRSEQUMAPAPU
      AND IDTIPOITMAPR      IN ('OC','OD')
      AND IDCARGATRIBUTARIA = 'S';

   IF P_BOIMPOSAPUR THEN
      V_IDAPURACAO := 'A';

      /******** Apurao Conjunta ********/
      UPDATE IMPOSTONF
         SET NRSEQMAPOIMP = P_NRSEQUMAPAPU
       WHERE NRSEQMAPOIMP IS NULL
         AND (CDFILIAL, NRLANCTONF, CDIMPOSTO) IN (SELECT IP.CDFILIAL, IP.NRLANCTONF, IP.CDIMPOSTO
                                                     FROM NOTAFISCAL N, IMPOSTONF IP
                                                    WHERE CDEMPRESA       = P_CDEMPRESA
                                                      AND IDSTATUS        IN ('A','E')
                                                      AND IP.CDFILIAL     = N.CDFILIAL
                                                      AND IP.NRLANCTONF   = N.NRLANCTONF
                                                      AND (((P_INSCRICAO IS NULL)     AND (P_IDORGARRECADNFE = 'F'  OR P_IDORGARRECADNFE IS NULL)) OR
                                                            (CDINSCESTA = P_INSCRICAO AND  P_IDORGARRECADNFE = 'E') OR
                                                            (CDINSCMUNI = P_INSCRICAO AND  P_IDORGARRECADNFE = 'M'))
                                                      AND (((P_IDNFFORAPERAPU='N') AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL)) OR
                                                           ((P_IDNFFORAPERAPU='S') AND (IP.DTREFMAPAPU IS NULL) AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL)) OR
                                                           ((P_IDNFFORAPERAPU='S') AND (IP.DTREFMAPAPU BETWEEN P_DTINICIAL AND P_DTFINAL) AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL)))
                                                      AND ((IP.CDIMPOSTO IN (SELECT CDIMPOSTO FROM IMPOSTO
                                                                              WHERE CDIMPOSTONFE = P_CDIMPOSTO)) OR
                                                           (IP.CDIMPOSTO IN (SELECT CDIMPOSTO FROM IMPOSTO
                                                                              WHERE CDIMPOSAPUR IN (SELECT CDIMPOSTO FROM IMPOSTO
                                                                                                     WHERE CDIMPOSTONFE = P_CDIMPOSTO)))));
   ELSE
      FOR C_IMPOSRELAC IN (SELECT CDIMPOSTO, CDIMPCARGATRIB
                             FROM IMPOSTO
                            WHERE CDIMPOSTONFE = P_CDIMPOSTO
                            ORDER BY CDIMPOSTO) LOOP

         /******** Carga Tributria ********/
         IF (C_IMPOSRELAC.CDIMPCARGATRIB IS NOT NULL) AND (V_IDUTCARGATRIB = 'S') THEN
            V_UTCARGATRIB := TRUE;

            IF (V_IDCREDDEBNORM = 'S') OR (V_IDOUTCREDDEB = 'S') THEN
               /******** Updata as notas do Imposto Normal ********/
               UPDATE IMPOSTONF
                  SET NRSEQUMAPAPU = P_NRSEQUMAPAPU
                WHERE NRSEQUMAPAPU IS NULL
                  AND (CDFILIAL, NRLANCTONF, CDIMPOSTO) IN (SELECT IP.CDFILIAL, IP.NRLANCTONF, IP.CDIMPOSTO
                                                              FROM NOTAFISCAL N, IMPOSTO I, IMPOSTONF IP
                                                             WHERE CDEMPRESA       = P_CDEMPRESA
                                                               AND IDSTATUS        IN ('A','E')
                                                               AND I.CDIMPOSTO     = C_IMPOSRELAC.CDIMPOSTO
                                                               AND IP.CDFILIAL     = N.CDFILIAL
                                                               AND IP.NRLANCTONF   = N.NRLANCTONF
                                                               AND (IP.CDIMPOSTO   = C_IMPOSRELAC.CDIMPOSTO OR
                                                                    IP.CDIMPOSTO   IN (SELECT CDIMPOSTO
                                                                                         FROM IMPOSTO
                                                                                        WHERE CDIMPOSAPUR  = C_IMPOSRELAC.CDIMPOSTO ))
                                                               AND (((P_INSCRICAO IS NULL)     AND (I.IDORGARRECAD = 'F'  OR I.IDORGARRECAD IS NULL)) OR
                                                                     (CDINSCESTA = P_INSCRICAO AND  I.IDORGARRECAD = 'E') OR
                                                                     (CDINSCMUNI = P_INSCRICAO AND  I.IDORGARRECAD = 'M'))
                                                               AND (((P_IDNFFORAPERAPU='N') AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL)) OR
                                                                    ((P_IDNFFORAPERAPU='S') AND (IP.DTREFMAPAPU IS NULL) AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL)) OR
                                                                    ((P_IDNFFORAPERAPU='S') AND (IP.DTREFMAPAPU BETWEEN P_DTINICIAL AND P_DTFINAL) AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL))));

               /******** Updata as notas do Imposto de Carga Tributria ********/
               UPDATE IMPOSTONF
                  SET NRSEQMAPOIMP = P_NRSEQUMAPAPU
                WHERE NRSEQMAPOIMP IS NULL
                  AND (CDFILIAL, NRLANCTONF, CDIMPOSTO) IN (SELECT IP.CDFILIAL, IP.NRLANCTONF, IP.CDIMPOSTO
                                                              FROM NOTAFISCAL N, IMPOSTO I, IMPOSTONF IP
                                                             WHERE CDEMPRESA       = P_CDEMPRESA
                                                               AND IDSTATUS        IN ('A','E')
                                                               AND I.CDIMPOSTO     = C_IMPOSRELAC.CDIMPOSTO
                                                               AND IP.CDFILIAL     = N.CDFILIAL
                                                               AND IP.NRLANCTONF   = N.NRLANCTONF
                                                               AND IP.CDIMPOSTO    = C_IMPOSRELAC.CDIMPCARGATRIB
                                                               AND (((P_INSCRICAO IS NULL)     AND (I.IDORGARRECAD = 'F' OR I.IDORGARRECAD IS NULL)) OR
                                                                     (CDINSCESTA = P_INSCRICAO AND  I.IDORGARRECAD = 'E') OR
                                                                     (CDINSCMUNI = P_INSCRICAO AND  I.IDORGARRECAD = 'M'))
                                                               AND (((P_IDNFFORAPERAPU='N') AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL)) OR
                                                                    ((P_IDNFFORAPERAPU='S') AND (IP.DTREFMAPAPU IS NULL ) AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL)) OR
                                                                    ((P_IDNFFORAPERAPU='S') AND (IP.DTREFMAPAPU BETWEEN P_DTINICIAL AND P_DTFINAL) AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL))));

               /******** Carga Tributria como Outros Crditos/Dbitos ********/
               IF V_IDOUTCREDDEB = 'S' THEN

                  /******** Totaliza os Valores de Entrada e Sada da Carga Tributria ********/
                  BEGIN
                     SELECT NVL(SUM(DECODE(N.IDENTRSAIDOP,'S',VRIMPCARGATRIBI,0)),0) AS VRCARGSAID,
                            NVL(SUM(DECODE(N.IDENTRSAIDOP,'E',VRIMPCARGATRIBI,0)),0) AS VRCARGENTR
                       INTO V_VRCARGSAID, V_VRCARGENTR
                       FROM IMPOSTONF I, NOTAFISCAL N
                      WHERE I.NRSEQUMAPAPU = P_NRSEQUMAPAPU
                        AND I.CDIMPOSTO    = P_CDIMPOSTO
                        AND I.CDFILIAL     = N.CDFILIAL
                        AND I.NRLANCTONF   = N.NRLANCTONF
                        AND NVL(N.IDSISTEMA,'DFS') <> 'FIS';
                  EXCEPTION
                     WHEN NO_DATA_FOUND THEN
                        V_VRCARGSAID := 0;
                        V_VRCARGENTR := 0;
                  END;

                  /******** Insere o item de Outros Dbitos ********/
                  IF V_VRCARGSAID > 0 THEN
                     V_NRSEQITMAPR := 0;

                     BEGIN
                        SELECT MAX(NRSEQITMAPR) AS NRSEQITMAPR
                          INTO V_NRSEQITMAPR
                          FROM ITMAPARECUP
                         WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
                           AND IDTIPOITMAPR = 'OD';
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_NRSEQITMAPR := 0;
                     END;

                     V_NRSEQITMAPR := NVL(V_NRSEQITMAPR,0) + 1;

                     INSERT INTO ITMAPARECUP
                        (NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR, DSITEMMAPR, VRITEMMAPR, CDITEMOCOR, DSITEMOCOR,
                         DSFLEGALOCOR, NRLANCTONF, CDFILIAL, CDAJUSTEITMAPA, CDAJUSTEITSEF, CDITPROCEDENCIA,
                         IDCREDPRESUMIDO, IDDIFALMANUAL, IDCARGATRIBUTARIA)
                     VALUES
                        (P_NRSEQUMAPAPU, 'OD', LPAD(V_NRSEQITMAPR,10,'0'), V_DSOUTDEBICARGA, V_VRCARGSAID, '', '', '',
                         '', '', V_CDAJSTDEBICARGA, V_CDAJUSTEITSEF_DEBI, V_CDITPROCEDENCIA_DEBI, 'N', 'S', 'S');
                  END IF;

                  /******** Insere o item de Outros Crditos ********/
                  IF V_VRCARGENTR > 0 THEN
                     V_NRSEQITMAPR := 0;

                     BEGIN
                        SELECT MAX(NRSEQITMAPR) AS NRSEQITMAPR
                          INTO V_NRSEQITMAPR
                          FROM ITMAPARECUP
                         WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
                           AND IDTIPOITMAPR = 'OC';
                     EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                           V_NRSEQITMAPR := 0;
                     END;

                     V_NRSEQITMAPR := NVL(V_NRSEQITMAPR,0) + 1;

                     INSERT INTO ITMAPARECUP
                        (NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR, DSITEMMAPR, VRITEMMAPR, CDITEMOCOR, DSITEMOCOR,
                         DSFLEGALOCOR, NRLANCTONF, CDFILIAL, CDAJUSTEITMAPA, CDAJUSTEITSEF, CDITPROCEDENCIA,
                         IDCREDPRESUMIDO, IDDIFALMANUAL, IDCARGATRIBUTARIA)
                     VALUES
                        (P_NRSEQUMAPAPU, 'OC', LPAD(V_NRSEQITMAPR,10,'0'), V_DSOUTCREDCARGA, V_VRCARGENTR, '', '', '',
                         '', '', V_CDAJSTCREDCARGA, V_CDAJUSTEITSEF_CRED, V_CDITPROCEDENCIA_CRED, 'N', 'S', 'S');
                  END IF;
               END IF;
            END IF;
         ELSE
            /******** Apurao Normal ********/
            UPDATE IMPOSTONF
               SET NRSEQUMAPAPU = P_NRSEQUMAPAPU
             WHERE NRSEQUMAPAPU IS NULL
               AND NVL(NRSEQMAPOIMP,' ') NOT IN (SELECT NRSEQUMAPAPU
                                                   FROM MAPAAPURAC
                                                  WHERE CDIMPOSTO = P_CDIMPOSTO
                                                    AND CDEMPRESA = P_CDEMPRESA
                                                    AND (((P_INSCRICAO IS NULL) AND (CDINSCESTA IS NULL) AND (CDINSCMUNI IS NULL)) OR
                                                         (CDINSCESTA = P_INSCRICAO) OR
                                                         (CDINSCMUNI = P_INSCRICAO)))
               AND (CDFILIAL, NRLANCTONF, CDIMPOSTO) IN (SELECT IP.CDFILIAL, IP.NRLANCTONF, IP.CDIMPOSTO
                                                           FROM NOTAFISCAL N, IMPOSTO I, IMPOSTONF IP
                                                          WHERE CDEMPRESA       = P_CDEMPRESA
                                                            AND IDSTATUS        IN ('A', 'E')
                                                            AND I.CDIMPOSTO     = C_IMPOSRELAC.CDIMPOSTO
                                                            AND IP.CDFILIAL     = N.CDFILIAL
                                                            AND IP.NRLANCTONF   = N.NRLANCTONF
                                                            AND (IP.CDIMPOSTO   = C_IMPOSRELAC.CDIMPOSTO OR
                                                                 IP.CDIMPOSTO   IN (SELECT CDIMPOSTO
                                                                                      FROM IMPOSTO
                                                                                     WHERE CDIMPOSAPUR  = C_IMPOSRELAC.CDIMPOSTO ))
                                                            AND (((P_INSCRICAO IS NULL)     AND (I.IDORGARRECAD = 'F'  OR I.IDORGARRECAD IS NULL)) OR
                                                                  (CDINSCESTA = P_INSCRICAO AND  I.IDORGARRECAD = 'E') OR
                                                                  (CDINSCMUNI = P_INSCRICAO AND  I.IDORGARRECAD = 'M'))
                                                            AND (((P_IDNFFORAPERAPU='N') AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL)) OR
                                                                 ((P_IDNFFORAPERAPU='S') AND (IP.DTREFMAPAPU IS NULL) AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL)) OR
                                                                 ((P_IDNFFORAPERAPU='S') AND (IP.DTREFMAPAPU BETWEEN P_DTINICIAL AND P_DTFINAL) AND (DECODE(IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND P_DTFINAL))));
         END IF;
      END LOOP;

      IF (V_IDAPURACAO IS NULL) AND (V_UTCARGATRIB) THEN
         V_IDAPURACAO := 'C';
      ELSE
         V_IDAPURACAO := 'N';
      END IF;
   END IF;

   /******** Documentos Fora do Perodo ********/
   V_NFFORAPERIODO := 'N';
   BEGIN
      SELECT MAX('S') AS NFFORAPERIODO
        INTO V_NFFORAPERIODO
        FROM NOTAFISCAL NF, IMPOSTONF IP
       WHERE ((IP.NRSEQUMAPAPU = P_NRSEQUMAPAPU) OR
              (IP.NRSEQMAPOIMP = P_NRSEQUMAPAPU))
         AND IP.CDFILIAL       = NF.CDFILIAL
         AND IP.NRLANCTONF     = NF.NRLANCTONF
         AND DECODE(NF.IDENTRSAIDOP,'E',DTENTRSAID,DTEMISSAO) BETWEEN P_DTININFFORPER AND (TO_DATE(P_DTINICIAL)-1);
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_NFFORAPERIODO := 'N';
   END;

   RETURN(V_IDAPURACAO || NVL(V_NFFORAPERIODO,'N') || NVL(V_IDOUTCREDDEB,'N'));
END;