import { Http, Metadata } from '@zeedhi/core';
import { Select } from '@zeedhi/common';
import { TekGrid, TekGridColumn } from '@zeedhi/teknisa-components-common';

Metadata.get('metadata/queryRunner.json');

export class QueryRunnerController {
  public query: string = '';

  public async runQuery() {
        const connection = Metadata.getInstance<Select>('conectionSelect');
        const grid = Metadata.getInstance<TekGrid>('queryRunnerGrid');
        const uri = '/runQuery';
        const requestBody = {requestType: "Row", row: {}};
        for (let i = 0; i < connection.datasource.data.length; i += 1) {
          if (connection.datasource.data[i]._id === connection.value) {
              const data = connection.datasource.data[i];
              requestBody.row = {changed: true,
              connectionDbUrl: data.dbUrl,
              connectionId: data._id,
              connectionName: data.name,
              connectionPassword: data.password,
              connectionPasswordSsl: data.passwordSsl,
              connectionUsername: data.username,
              driverClass: data.driverClass,
              oracleWalletUse: data.oracleWalletUse,
              query: this.query,
              room: data.room
              };
          }
        }
        grid.datasource.loading = true;
        await Http.post(uri, requestBody).then(function (queryResult) {
           const columns = queryResult.data.dataset.queryResult;
           const newColumns = Object.keys(columns[0]);
           const tekgridcolumns = newColumns.map((item: any) => (new TekGridColumn({name: item, label: item, sortable: false}, grid)));
           grid.columns = tekgridcolumns;
           grid.datasource.updateData(columns);
           grid.datasource.loading = false;
           grid.reload();
        });
    }
}
